@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import os
+import random
 import re
 import string
 import subprocess
@@ -190,7 +191,7 @@ git_check(
 main_commits = [
     line.split("\t")
     for line in git_output(
-        f'log -{HISTORY_DEPTH} --pretty="format:%h\t%s" {source_remote}/{backport_target}..{source_remote}/main'
+        f'log -{HISTORY_DEPTH} --abbrev=12 --pretty="format:%h\t%s" {source_remote}/{backport_target}..{source_remote}/main'
     ).splitlines()
     if line
 ]
@@ -200,7 +201,7 @@ print(f"Have {len(main_commits)} new commits in the main branch.")
 branch_commits = [
     line.split("\t")
     for line in git_output(
-        f'log -{HISTORY_DEPTH} --pretty="format:%h\t%s" {source_remote}/main..{source_remote}/{backport_target}'
+        f'log -{HISTORY_DEPTH} --abbrev=12 --pretty="format:%h\t%s" {source_remote}/main..{source_remote}/{backport_target}'
     ).splitlines()
     if line
 ]
@@ -412,9 +413,12 @@ print(
 git_check(f"fetch {source_remote}")
 
 # Now, go over the list of PRs that we have collected, and try to backport
-# each of them.
+# each of them. Do it in randomized order to avoid getting stuck on a single
+# error.
 print(f"Have {len(prs_to_backport)} PRs to backport.")
-for index, pr_info in enumerate(prs_to_backport.values()):
+for index, pr_info in enumerate(
+    random.sample(list(prs_to_backport.values()), len(prs_to_backport))
+):
     print()
 
     # Don't want to have an endless loop that modifies the repository in an
@@ -542,7 +546,7 @@ for index, pr_info in enumerate(prs_to_backport.values()):
     original_description = re.sub(
         r"((fix|clos|resolv)[esd]+)(\s+#[0-9]+)",
         r"`\1`\3",
-        original_pr.body,
+        original_pr.body or "",  # Match "" if pr_body is None
         flags=re.IGNORECASE,
     )
     backport_description += (
