@@ -21,6 +21,7 @@
 #include <utils/palloc.h>
 #include <utils/rel.h>
 #include <utils/relcache.h>
+#include <utils/snapmgr.h>
 #include <utils/syscache.h>
 
 #include "compat/compat.h"
@@ -277,12 +278,10 @@ ts_chunk_constraint_dimensional_create(const Dimension *dim, const DimensionSlic
 									   const char *name)
 {
 	Constraint *constr = NULL;
-	bool isvarlena;
 	Node *dimdef;
 	ColumnRef *colref;
-	Datum startdat, enddat;
 	List *compexprs = NIL;
-	Oid outfuncid;
+	Oid type;
 
 	if (slice->fd.range_start == PG_INT64_MIN && slice->fd.range_end == PG_INT64_MAX)
 		return NULL;
@@ -307,16 +306,12 @@ ts_chunk_constraint_dimensional_create(const Dimension *dim, const DimensionSlic
 			/* The dimension has a time function to compute the time value so
 			 * need to convert the range values to the time type returned by
 			 * the partitioning function. */
-			getTypeOutputInfo(partinfo->partfunc.rettype, &outfuncid, &isvarlena);
-			startdat = ts_internal_to_time_value(slice->fd.range_start, partinfo->partfunc.rettype);
-			enddat = ts_internal_to_time_value(slice->fd.range_end, partinfo->partfunc.rettype);
+			type = partinfo->partfunc.rettype;
 		}
 		else
 		{
-			/* Closed dimension, just use the integer output function */
-			getTypeOutputInfo(INT8OID, &outfuncid, &isvarlena);
-			startdat = Int64GetDatum(slice->fd.range_start);
-			enddat = Int64GetDatum(slice->fd.range_end);
+			/* Closed dimension, just use the INT8 type */
+			type = INT8OID;
 		}
 	}
 	else
@@ -325,28 +320,24 @@ ts_chunk_constraint_dimensional_create(const Dimension *dim, const DimensionSlic
 		Assert(IS_OPEN_DIMENSION(dim));
 
 		dimdef = (Node *) colref;
-		getTypeOutputInfo(dim->fd.column_type, &outfuncid, &isvarlena);
-		startdat = ts_internal_to_time_value(slice->fd.range_start, dim->fd.column_type);
-		enddat = ts_internal_to_time_value(slice->fd.range_end, dim->fd.column_type);
+		type = dim->fd.column_type;
 	}
 
 	/*
-	 * Convert internal format datums to string (output) datums.
-	 *
 	 * We are forcing ISO datestyle here to prevent parsing errors with
 	 * certain timezone/datestyle combinations.
 	 */
 	int current_datestyle = DateStyle;
 	DateStyle = USE_ISO_DATES;
-	startdat = OidFunctionCall1(outfuncid, startdat);
-	enddat = OidFunctionCall1(outfuncid, enddat);
+	char *start_str = ts_internal_to_time_string(slice->fd.range_start, type);
+	char *end_str = ts_internal_to_time_string(slice->fd.range_end, type);
 	DateStyle = current_datestyle;
 
 	/* Elide range constraint for +INF or -INF */
 	if (slice->fd.range_start != PG_INT64_MIN)
 	{
 		A_Const *start_const = makeNode(A_Const);
-		memcpy(&start_const->val, makeString(DatumGetCString(startdat)), sizeof(start_const->val));
+		memcpy(&start_const->val, makeString(start_str), sizeof(start_const->val));
 		start_const->location = -1;
 		A_Expr *ge_expr = makeSimpleA_Expr(AEXPR_OP, ">=", dimdef, (Node *) start_const, -1);
 		compexprs = lappend(compexprs, ge_expr);
@@ -355,7 +346,7 @@ ts_chunk_constraint_dimensional_create(const Dimension *dim, const DimensionSlic
 	if (slice->fd.range_end != PG_INT64_MAX)
 	{
 		A_Const *end_const = makeNode(A_Const);
-		memcpy(&end_const->val, makeString(DatumGetCString(enddat)), sizeof(end_const->val));
+		memcpy(&end_const->val, makeString(end_str), sizeof(end_const->val));
 		end_const->location = -1;
 		A_Expr *lt_expr = makeSimpleA_Expr(AEXPR_OP, "<", dimdef, (Node *) end_const, -1);
 		compexprs = lappend(compexprs, lt_expr);
@@ -367,6 +358,9 @@ ts_chunk_constraint_dimensional_create(const Dimension *dim, const DimensionSlic
 	constr->deferrable = false;
 	constr->skip_validation = true;
 	constr->initially_valid = true;
+#if PG18_GE
+	constr->is_enforced = true;
+#endif
 
 	Assert(list_length(compexprs) >= 1);
 
@@ -416,36 +410,22 @@ create_non_dimensional_constraint(const ChunkConstraint *cc, Oid chunk_oid, int3
 
 	Assert(!is_dimension_constraint(cc));
 
-	ts_process_utility_set_expect_chunk_modification(true);
-	chunk_constraint_oid = chunk_constraint_create_on_table(cc, chunk_oid);
-	ts_process_utility_set_expect_chunk_modification(false);
-
 	/*
-	 * The table constraint might not have been created if this constraint
-	 * corresponds to a dimension slice that covers the entire range of values
-	 * in the particular dimension. In that case, there is no need to add a
-	 * table constraint.
+	 * If we're creating constraints for a new chunk from an existing
+	 * table or attaching a chunk to an existing hypertable, we might
+	 * have constraints already created. If so, skip creating
+	 * such constraints, we only needed their metadata to be added.
 	 */
-	if (!OidIsValid(chunk_constraint_oid))
-		return InvalidOid;
-
-	Oid hypertable_constraint_oid =
-		get_relation_constraint_oid(hypertable_oid,
-									NameStr(cc->fd.hypertable_constraint_name),
-									false);
-	HeapTuple tuple = SearchSysCache1(CONSTROID, hypertable_constraint_oid);
-
-	if (HeapTupleIsValid(tuple))
+	if (ConstraintNameIsUsed(CONSTRAINT_RELATION, chunk_oid, NameStr(cc->fd.constraint_name)))
 	{
-		FormData_pg_constraint *constr = (FormData_pg_constraint *) GETSTRUCT(tuple);
-
-		if (OidIsValid(constr->conindid) && constr->contype != CONSTRAINT_FOREIGN)
-			ts_chunk_index_create_from_constraint(hypertable_id,
-												  hypertable_constraint_oid,
-												  chunk_id,
-												  chunk_constraint_oid);
-
-		ReleaseSysCache(tuple);
+		chunk_constraint_oid =
+			get_relation_constraint_oid(chunk_oid, NameStr(cc->fd.constraint_name), true);
+	}
+	else
+	{
+		ts_process_utility_set_expect_chunk_modification(true);
+		chunk_constraint_oid = chunk_constraint_create_on_table(cc, chunk_oid);
+		ts_process_utility_set_expect_chunk_modification(false);
 	}
 
 	return chunk_constraint_oid;
@@ -489,6 +469,7 @@ ts_chunk_constraints_create(const Hypertable *ht, const Chunk *chunk)
 
 			dim = ts_hyperspace_get_dimension_by_id(ht->space, slice->fd.dimension_id);
 			Assert(dim);
+
 			constr =
 				ts_chunk_constraint_dimensional_create(dim, slice, NameStr(cc->fd.constraint_name));
 
@@ -777,6 +758,7 @@ typedef struct ConstraintContext
 	char chunk_relkind;
 	ChunkConstraints *ccs;
 	int32 chunk_id;
+	Oid chunk_relid;
 } ConstraintContext;
 
 static ConstraintProcessStatus
@@ -787,7 +769,19 @@ chunk_constraint_add(HeapTuple constraint_tuple, void *arg)
 
 	if (cc->chunk_relkind != RELKIND_FOREIGN_TABLE && chunk_constraint_need_on_chunk(constraint))
 	{
-		ts_chunk_constraints_add(cc->ccs, cc->chunk_id, 0, NULL, NameStr(constraint->conname));
+		/* If the chunk already has an equivalent constraint, use the existing one. */
+		Relation chunk = table_open(cc->chunk_relid, AccessShareLock);
+		Form_pg_constraint matching_const = ts_constraint_find_matching(constraint_tuple, chunk);
+		table_close(chunk, NoLock);
+
+		if (matching_const != NULL)
+			ts_chunk_constraints_add(cc->ccs,
+									 cc->chunk_id,
+									 0,
+									 NameStr(matching_const->conname),
+									 NameStr(constraint->conname));
+		else
+			ts_chunk_constraints_add(cc->ccs, cc->chunk_id, 0, NULL, NameStr(constraint->conname));
 		return CONSTR_PROCESSED;
 	}
 
@@ -796,12 +790,17 @@ chunk_constraint_add(HeapTuple constraint_tuple, void *arg)
 
 int
 ts_chunk_constraints_add_inheritable_constraints(ChunkConstraints *ccs, int32 chunk_id,
-												 const char chunk_relkind, Oid hypertable_oid)
+												 const char chunk_relkind, Oid hypertable_oid,
+												 Oid table_id)
 {
+	/* This should never be called with NULL ccs.  */
+	Ensure(ccs, "ccs must not be NULL");
+
 	ConstraintContext cc = {
 		.chunk_relkind = chunk_relkind,
 		.ccs = ccs,
 		.chunk_id = chunk_id,
+		.chunk_relid = table_id,
 	};
 
 	return ts_constraint_process(hypertable_oid, chunk_constraint_add, &cc);
@@ -881,33 +880,6 @@ hypertable_constraint_matches_tuple(TupleInfo *ti, const char *hypertable_constr
 	return !isnull && namestrcmp(DatumGetName(name), hypertable_constraint_name) == 0;
 }
 
-static void
-chunk_constraint_delete_metadata(TupleInfo *ti)
-{
-	bool isnull;
-	Datum constrname = slot_getattr(ti->slot, Anum_chunk_constraint_constraint_name, &isnull);
-	int32 chunk_id = DatumGetInt32(slot_getattr(ti->slot, Anum_chunk_constraint_chunk_id, &isnull));
-	/* Get the chunk relid. Note that, at this point, the chunk table can be
-	 * deleted already */
-	Oid chunk_relid = ts_chunk_get_relid(chunk_id, true);
-
-	if (OidIsValid(chunk_relid))
-	{
-		Oid index_relid = get_constraint_index(
-			get_relation_constraint_oid(chunk_relid, NameStr(*DatumGetName(constrname)), true));
-
-		/*
-		 * If this is an index constraint, we need to cleanup the index
-		 * metadata. Don't drop the index though, since that will happen when
-		 * the constraint is dropped.
-		 */
-		if (OidIsValid(index_relid))
-			ts_chunk_index_delete(chunk_id, get_rel_name(index_relid), false);
-	}
-
-	ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
-}
-
 static void
 chunk_constraint_drop_constraint(TupleInfo *ti)
 {
@@ -927,14 +899,14 @@ chunk_constraint_drop_constraint(TupleInfo *ti)
 		};
 
 		if (OidIsValid(constrobj.objectId))
-			performDeletion(&constrobj, DROP_RESTRICT, 0);
+			/* must use DROP_CASCADE if regular table references a hypertable */
+			performDeletion(&constrobj, DROP_CASCADE, 0);
 	}
 }
 
 int
 ts_chunk_constraint_delete_by_hypertable_constraint_name(int32 chunk_id,
-														 const char *hypertable_constraint_name,
-														 bool delete_metadata, bool drop_constraint)
+														 const char *hypertable_constraint_name)
 {
 	ScanIterator iterator =
 		ts_scan_iterator_create(CHUNK_CONSTRAINT, RowExclusiveLock, CurrentMemoryContext);
@@ -950,18 +922,15 @@ ts_chunk_constraint_delete_by_hypertable_constraint_name(int32 chunk_id,
 
 		count++;
 
-		if (delete_metadata)
-			chunk_constraint_delete_metadata(ts_scan_iterator_tuple_info(&iterator));
-
-		if (drop_constraint)
-			chunk_constraint_drop_constraint(ts_scan_iterator_tuple_info(&iterator));
+		TupleInfo *ti = ts_scan_iterator_tuple_info(&iterator);
+		ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
+		chunk_constraint_drop_constraint(ti);
 	}
 	return count;
 }
 
 int
-ts_chunk_constraint_delete_by_constraint_name(int32 chunk_id, const char *constraint_name,
-											  bool delete_metadata, bool drop_constraint)
+ts_chunk_constraint_delete_by_constraint_name(int32 chunk_id, const char *constraint_name)
 {
 	ScanIterator iterator =
 		ts_scan_iterator_create(CHUNK_CONSTRAINT, RowExclusiveLock, CurrentMemoryContext);
@@ -971,11 +940,8 @@ ts_chunk_constraint_delete_by_constraint_name(int32 chunk_id, const char *constr
 	ts_scanner_foreach(&iterator)
 	{
 		count++;
-		if (delete_metadata)
-			chunk_constraint_delete_metadata(ts_scan_iterator_tuple_info(&iterator));
-
-		if (drop_constraint)
-			chunk_constraint_drop_constraint(ts_scan_iterator_tuple_info(&iterator));
+		TupleInfo *ti = ts_scan_iterator_tuple_info(&iterator);
+		ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
 	}
 	return count;
 }
@@ -984,7 +950,7 @@ ts_chunk_constraint_delete_by_constraint_name(int32 chunk_id, const char *constr
  * Delete all constraints for a chunk. Optionally, collect the deleted constraints.
  */
 int
-ts_chunk_constraint_delete_by_chunk_id(int32 chunk_id, ChunkConstraints *ccs)
+ts_chunk_constraint_delete_by_chunk_id(int32 chunk_id, ChunkConstraints *ccs, bool drop_constraint)
 {
 	ScanIterator iterator =
 		ts_scan_iterator_create(CHUNK_CONSTRAINT, RowExclusiveLock, CurrentMemoryContext);
@@ -997,8 +963,11 @@ ts_chunk_constraint_delete_by_chunk_id(int32 chunk_id, ChunkConstraints *ccs)
 		count++;
 
 		ts_chunk_constraints_add_from_tuple(ccs, ts_scan_iterator_tuple_info(&iterator));
-		chunk_constraint_delete_metadata(ts_scan_iterator_tuple_info(&iterator));
-		chunk_constraint_drop_constraint(ts_scan_iterator_tuple_info(&iterator));
+		TupleInfo *ti = ts_scan_iterator_tuple_info(&iterator);
+		ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
+
+		if (drop_constraint)
+			chunk_constraint_drop_constraint(ts_scan_iterator_tuple_info(&iterator));
 	}
 	return count;
 }
@@ -1016,8 +985,9 @@ ts_chunk_constraint_delete_by_dimension_slice_id(int32 dimension_slice_id)
 	{
 		count++;
 
-		chunk_constraint_delete_metadata(ts_scan_iterator_tuple_info(&iterator));
-		chunk_constraint_drop_constraint(ts_scan_iterator_tuple_info(&iterator));
+		TupleInfo *ti = ts_scan_iterator_tuple_info(&iterator);
+		ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
+		chunk_constraint_drop_constraint(ti);
 	}
 	return count;
 }
@@ -1094,12 +1064,6 @@ chunk_constraint_rename_hypertable_from_tuple(TupleInfo *ti, const char *new_nam
 										   NameStr(new_chunk_constraint_name));
 
 	new_tuple = heap_modify_tuple(tuple, tupdesc, values, nulls, doReplace);
-
-	ts_chunk_index_adjust_meta(chunk_id,
-							   NameStr(new_hypertable_constraint_name),
-							   NameStr(*old_chunk_constraint_name),
-							   NameStr(new_chunk_constraint_name));
-
 	ts_catalog_update(ti->scanrel, new_tuple);
 	heap_freetuple(new_tuple);
 
@@ -1111,14 +1075,14 @@ chunk_constraint_rename_hypertable_from_tuple(TupleInfo *ti, const char *new_nam
  * Adjust internal metadata after index/constraint rename
  */
 int
-ts_chunk_constraint_adjust_meta(int32 chunk_id, const char *ht_constraint_name,
-								const char *old_name, const char *new_name)
+ts_chunk_constraint_adjust_meta(int32 chunk_id, const char *ht_name, const char *chunk_old_name,
+								const char *chunk_new_name)
 {
 	ScanIterator iterator =
 		ts_scan_iterator_create(CHUNK_CONSTRAINT, RowExclusiveLock, CurrentMemoryContext);
 	int count = 0;
 
-	init_scan_by_chunk_id_constraint_name(&iterator, chunk_id, old_name);
+	init_scan_by_chunk_id_constraint_name(&iterator, chunk_id, chunk_old_name);
 
 	ts_scanner_foreach(&iterator)
 	{
@@ -1138,9 +1102,9 @@ ts_chunk_constraint_adjust_meta(int32 chunk_id, const char *ht_constraint_name,
 		 * after them.
 		 */
 		NameData ht_constraint_namedata;
-		namestrcpy(&ht_constraint_namedata, ht_constraint_name);
+		namestrcpy(&ht_constraint_namedata, ht_name);
 		NameData new_namedata;
-		namestrcpy(&new_namedata, new_name);
+		namestrcpy(&new_namedata, chunk_new_name);
 
 		values[AttrNumberGetAttrOffset(Anum_chunk_constraint_hypertable_constraint_name)] =
 			NameGetDatum(&ht_constraint_namedata);
@@ -1265,3 +1229,132 @@ ts_chunk_constraint_get_name_from_hypertable_constraint(Oid chunk_relid,
 	}
 	return NULL;
 }
+
+int
+ts_chunk_constraint_delete_dimensional_constraints(int32 chunk_id, ChunkConstraints *ccs)
+{
+	ScanIterator iterator =
+		ts_scan_iterator_create(CHUNK_CONSTRAINT, RowExclusiveLock, CurrentMemoryContext);
+	int count = 0;
+
+	ts_chunk_constraint_scan_iterator_set_chunk_id(&iterator, chunk_id);
+
+	ts_scanner_foreach(&iterator)
+	{
+		TupleInfo *ti = ts_scan_iterator_tuple_info(&iterator);
+		bool isnull;
+		int32 slice_id = DatumGetInt32(
+			slot_getattr(ti->slot, Anum_chunk_constraint_dimension_slice_id, &isnull));
+
+		if (isnull || slice_id == 0)
+			continue;
+
+		count++;
+
+		ts_chunk_constraints_add_from_tuple(ccs, ti);
+		ts_catalog_delete_tid(ti->scanrel, ts_scanner_get_tuple_tid(ti));
+		chunk_constraint_drop_constraint(ti);
+	}
+	return count;
+}
+
+/*
+ * Drop a constraint using a pg_constraint heap tuple.
+ */
+void
+ts_chunk_constraint_drop_from_tuple(HeapTuple constraint_tuple)
+{
+	FormData_pg_constraint *constr = (FormData_pg_constraint *) GETSTRUCT(constraint_tuple);
+	ObjectAddress constrobj = {
+		.classId = ConstraintRelationId,
+		.objectId = constr->oid,
+	};
+
+	if (OidIsValid(constr->conparentid))
+	{
+		deleteDependencyRecordsForClass(constrobj.classId,
+										constrobj.objectId,
+										ConstraintRelationId,
+										DEPENDENCY_INTERNAL);
+		CommandCounterIncrement();
+	}
+
+	if (OidIsValid(constrobj.objectId))
+		performDeletion(&constrobj, DROP_RESTRICT, 0);
+}
+
+static void
+check_chunk_constraint_violated(Oid chunk_relid, const Dimension *dim, const DimensionSlice *slice)
+{
+	Relation rel;
+	TupleTableSlot *slot;
+	TableScanDesc scandesc;
+	bool isnull;
+	int attno = get_attnum(chunk_relid, NameStr(dim->fd.column_name));
+	Ensure(attno != InvalidAttrNumber, "invalid attribute number");
+
+	PushActiveSnapshot(GetLatestSnapshot());
+	rel = table_open(chunk_relid, AccessShareLock);
+	scandesc = table_beginscan(rel, GetActiveSnapshot(), 0, NULL);
+	slot = table_slot_create(rel, NULL);
+
+	while (table_scan_getnextslot(scandesc, ForwardScanDirection, slot))
+	{
+		Datum datum;
+		int64 value;
+
+		datum = slot_getattr(slot, attno, &isnull);
+		Assert(!isnull);
+
+		if (NULL != dim->partitioning)
+		{
+			Oid collation = TupleDescAttr(slot->tts_tupleDescriptor, AttrNumberGetAttrOffset(attno))
+								->attcollation;
+			datum = ts_partitioning_func_apply(dim->partitioning, collation, datum);
+		}
+
+		if (dim->type == DIMENSION_TYPE_OPEN)
+			value = ts_time_value_to_internal(datum, ts_dimension_get_partition_type(dim));
+		else if (dim->type == DIMENSION_TYPE_CLOSED)
+			value = (int64) DatumGetInt32(datum);
+		else
+			elog(ERROR, "invalid dimension type when checking constraint");
+
+		if (value < slice->fd.range_start || value >= slice->fd.range_end)
+			ereport(ERROR,
+					(errcode(ERRCODE_CHECK_VIOLATION),
+					 errmsg("dimension constraint for column \"%s\" violated by some row",
+							NameStr(dim->fd.column_name))));
+	}
+
+	ExecDropSingleTupleTableSlot(slot);
+	table_endscan(scandesc);
+	table_close(rel, NoLock);
+	PopActiveSnapshot();
+}
+
+/*
+ * Check whether the chunk has any row that violates any of its dimensional constraints
+ */
+void
+ts_chunk_constraint_check_violated(const Chunk *chunk, const Hyperspace *hs)
+{
+	const ChunkConstraints *ccs = chunk->constraints;
+
+	for (int i = 0; i < ccs->num_constraints; i++)
+	{
+		const ChunkConstraint *cc = &ccs->constraints[i];
+
+		if (is_dimension_constraint(cc))
+		{
+			const DimensionSlice *slice = get_slice_with_id(chunk->cube, cc->fd.dimension_slice_id);
+			const Dimension *dim;
+
+			dim = ts_hyperspace_get_dimension_by_id(hs, slice->fd.dimension_id);
+			Assert(dim);
+
+			/* Check if the chunk has any row that violates the constraint */
+			check_chunk_constraint_violated(chunk->table_id, dim, slice);
+		}
+	}
+}
