@@ -10,13 +10,17 @@
  */
 
 #include <postgres.h>
+
 #include <access/htup_details.h>
 #include <catalog/dependency.h>
 #include <catalog/namespace.h>
 #include <catalog/pg_trigger.h>
 #include <commands/trigger.h>
+#include <executor/spi.h>
 #include <fmgr.h>
+#include <lib/stringinfo.h>
 #include <nodes/makefuncs.h>
+#include <replication/slot.h>
 #include <storage/lmgr.h>
 #include <utils/acl.h>
 #include <utils/builtins.h>
@@ -46,6 +50,24 @@
 #define BUCKET_FUNCTION_SERIALIZE_VERSION 1
 #define CHECK_NAME_MATCH(name1, name2) (namestrcmp(name1, name2) == 0)
 
+TS_FUNCTION_INFO_V1(ts_invalidation_plugin_name);
+TS_FUNCTION_INFO_V1(ts_has_invalidation_trigger);
+
+/*
+ * Return the full name of the invalidation plugin, with version and all.
+ */
+Datum
+ts_invalidation_plugin_name(PG_FUNCTION_ARGS)
+{
+	PG_RETURN_TEXT_P(cstring_to_text(CONTINUOUS_AGGS_HYPERTABLE_INVALIDATION_PLUGIN_NAME));
+}
+
+Datum
+ts_has_invalidation_trigger(PG_FUNCTION_ARGS)
+{
+	PG_RETURN_BOOL(has_invalidation_trigger(PG_GETARG_OID(0)));
+}
+
 static void
 init_scan_by_mat_hypertable_id(ScanIterator *iterator, const int32 mat_hypertable_id)
 {
@@ -193,6 +215,12 @@ hypertable_invalidation_log_delete(int32 raw_hypertable_id)
 	}
 }
 
+void
+ts_get_invalidation_replication_slot_name(char *slotname, Size szslot)
+{
+	snprintf(slotname, szslot, "ts_%u_cagg", MyDatabaseId);
+}
+
 void
 ts_materialization_invalidation_log_delete_inner(int32 mat_hypertable_id)
 {
@@ -309,7 +337,7 @@ continuous_agg_formdata_fill(FormData_continuous_agg *fd, const TupleInfo *ti)
  * Fill the fields of a integer based bucketing function
  */
 static void
-cagg_fill_bucket_function_integer_based(ContinuousAggsBucketFunction *bf, bool *isnull,
+cagg_fill_bucket_function_integer_based(ContinuousAggBucketFunction *bf, bool *isnull,
 										Datum *values)
 {
 	/* Bucket width */
@@ -340,7 +368,7 @@ cagg_fill_bucket_function_integer_based(ContinuousAggsBucketFunction *bf, bool *
  * Fill the fields of a time based bucketing function
  */
 static void
-cagg_fill_bucket_function_time_based(ContinuousAggsBucketFunction *bf, bool *isnull, Datum *values)
+cagg_fill_bucket_function_time_based(ContinuousAggBucketFunction *bf, bool *isnull, Datum *values)
 {
 	/*
 	 * bucket_width
@@ -388,7 +416,7 @@ cagg_fill_bucket_function_time_based(ContinuousAggsBucketFunction *bf, bool *isn
 }
 
 static void
-continuous_agg_fill_bucket_function(int32 mat_hypertable_id, ContinuousAggsBucketFunction *bf)
+continuous_agg_fill_bucket_function(int32 mat_hypertable_id, ContinuousAggBucketFunction *bf)
 {
 	ScanIterator iterator;
 	int count = 0;
@@ -472,14 +500,14 @@ continuous_agg_init(ContinuousAgg *cagg, const Form_continuous_agg fd)
 	Assert(OidIsValid(cagg->relid));
 	Assert(OidIsValid(cagg->partition_type));
 
-	cagg->bucket_function = palloc0(sizeof(ContinuousAggsBucketFunction));
+	cagg->bucket_function = palloc0(sizeof(ContinuousAggBucketFunction));
 	continuous_agg_fill_bucket_function(cagg->data.mat_hypertable_id, cagg->bucket_function);
 }
 
-TSDLLEXPORT CaggsInfo
+TSDLLEXPORT ContinuousAggInfo
 ts_continuous_agg_get_all_caggs_info(int32 raw_hypertable_id)
 {
-	CaggsInfo all_caggs_info;
+	ContinuousAggInfo all_caggs_info;
 
 	List *caggs = ts_continuous_aggs_find_by_raw_table_id(raw_hypertable_id);
 	ListCell *lc;
@@ -502,6 +530,37 @@ ts_continuous_agg_get_all_caggs_info(int32 raw_hypertable_id)
 	return all_caggs_info;
 }
 
+/*
+ * Return true if there is any continuous aggregate that is using WAL-based
+ * invalidation collection.
+ *
+ * A hypertable is using the WAL-based invalidation collection if it has a
+ * attached continuous aggregate but does not have an invalidation trigger.
+ */
+static bool
+hypertable_invalidation_slot_used(void)
+{
+	ScanIterator iterator =
+		ts_scan_iterator_create(CONTINUOUS_AGG, AccessShareLock, CurrentMemoryContext);
+	ts_scanner_foreach(&iterator)
+	{
+		bool isnull;
+		Datum datum = slot_getattr(ts_scan_iterator_slot(&iterator),
+								   Anum_continuous_agg_raw_hypertable_id,
+								   &isnull);
+
+		Assert(!isnull);
+		Oid relid = ts_hypertable_id_to_relid(DatumGetInt32(datum), true);
+		if (!has_invalidation_trigger(relid))
+		{
+			ts_scan_iterator_close(&iterator);
+			return true;
+		}
+	}
+	ts_scan_iterator_close(&iterator);
+	return false;
+}
+
 TSDLLEXPORT ContinuousAggHypertableStatus
 ts_continuous_agg_hypertable_status(int32 hypertable_id)
 {
@@ -829,6 +888,10 @@ drop_continuous_agg(FormData_continuous_agg *cadata, bool drop_user_view)
 	 *
 	 * AccessExclusiveLock is needed to drop triggers and also prevent
 	 * concurrent DML commands.
+	 *
+	 * It is needed also in the case that we are using WAL-based invalidation
+	 * collection since we want to serialize create and drop of continuous
+	 * aggregates.
 	 */
 	if (drop_user_view)
 		user_view = get_and_lock_rel_by_name(&cadata->user_view_schema,
@@ -856,17 +919,13 @@ drop_continuous_agg(FormData_continuous_agg *cadata, bool drop_user_view)
 		LockRelationOid(catalog_get_table_id(catalog, CONTINUOUS_AGGS_INVALIDATION_THRESHOLD),
 						RowExclusiveLock);
 
-		/* The trigger will be dropped if the hypertable still exists and no other
+		/* The trigger will be dropped if it exists (it does not for WAL-based
+		 * invalidation collection), the hypertable still exists and no other
 		 * caggs attached. */
-		if (OidIsValid(raw_hypertable.objectId))
+		Oid tgoid = get_trigger_oid(raw_hypertable.objectId, CAGGINVAL_TRIGGER_NAME, true);
+		if (OidIsValid(raw_hypertable.objectId) && OidIsValid(tgoid))
 		{
-			ObjectAddressSet(raw_hypertable_trig,
-							 TriggerRelationId,
-							 get_trigger_oid(raw_hypertable.objectId,
-											 CAGGINVAL_TRIGGER_NAME,
-											 false));
-
-			/* Raw hypertable is locked above */
+			ObjectAddressSet(raw_hypertable_trig, TriggerRelationId, tgoid);
 			LockRelationOid(raw_hypertable_trig.objectId, AccessExclusiveLock);
 		}
 	}
@@ -924,6 +983,18 @@ drop_continuous_agg(FormData_continuous_agg *cadata, bool drop_user_view)
 		ts_hypertable_drop_trigger(raw_hypertable.objectId, CAGGINVAL_TRIGGER_NAME);
 	}
 
+	/*
+	 * Drop invalidation slot if there are no hypertables using WAL-based
+	 * invalidation collection.
+	 *
+	 * This is important since there is no actor that reads the slot, which
+	 * means that the WAL cannot be pruned.
+	 */
+	char slot_name[TS_INVALIDATION_SLOT_NAME_MAX];
+	ts_get_invalidation_replication_slot_name(slot_name, sizeof(slot_name));
+	if (!hypertable_invalidation_slot_used() && SearchNamedReplicationSlot(slot_name, true) != NULL)
+		ts_hypertable_drop_invalidation_replication_slot(slot_name);
+
 	if (OidIsValid(mat_hypertable.objectId))
 	{
 		performDeletion(&mat_hypertable, DROP_CASCADE, 0);
@@ -1297,11 +1368,11 @@ ts_continuous_agg_bucket_on_interval(Oid bucket_function)
 
 /*
  * Calls the desired time bucket function depending on the arguments. If the experimental flag is
- * set on ContinuousAggsBucketFunction, one of time_bucket_ng() versions is used. This is a common
+ * set on ContinuousAggBucketFunction, one of time_bucket_ng() versions is used. This is a common
  * procedure used by ts_compute_* below.
  */
 static Datum
-generic_time_bucket(const ContinuousAggsBucketFunction *bf, Datum timestamp)
+generic_time_bucket(const ContinuousAggBucketFunction *bf, Datum timestamp)
 {
 	FuncInfo *func_info = ts_func_cache_get_bucketing_func(bf->bucket_function);
 	Ensure(func_info != NULL, "unable to get bucket function for Oid %d", bf->bucket_function);
@@ -1395,7 +1466,7 @@ generic_time_bucket(const ContinuousAggsBucketFunction *bf, Datum timestamp)
  * Otherwise, it happens in UTC.
  */
 static Datum
-generic_add_interval(const ContinuousAggsBucketFunction *bf, Datum timestamp)
+generic_add_interval(const ContinuousAggBucketFunction *bf, Datum timestamp)
 {
 	Datum tzname = 0;
 	bool has_timezone = (bf->bucket_time_timezone != NULL);
@@ -1436,7 +1507,7 @@ generic_add_interval(const ContinuousAggsBucketFunction *bf, Datum timestamp)
  */
 void
 ts_compute_inscribed_bucketed_refresh_window_variable(int64 *start, int64 *end,
-													  const ContinuousAggsBucketFunction *bf)
+													  const ContinuousAggBucketFunction *bf)
 {
 	Datum start_old, end_old, start_aligned, end_aliged;
 
@@ -1474,7 +1545,7 @@ ts_compute_inscribed_bucketed_refresh_window_variable(int64 *start, int64 *end,
  */
 void
 ts_compute_circumscribed_bucketed_refresh_window_variable(int64 *start, int64 *end,
-														  const ContinuousAggsBucketFunction *bf)
+														  const ContinuousAggBucketFunction *bf)
 {
 	Datum start_old, end_old, start_new, end_new;
 
@@ -1505,7 +1576,7 @@ ts_compute_circumscribed_bucketed_refresh_window_variable(int64 *start, int64 *e
  */
 int64
 ts_compute_beginning_of_the_next_bucket_variable(int64 timeval,
-												 const ContinuousAggsBucketFunction *bf)
+												 const ContinuousAggBucketFunction *bf)
 {
 	Datum val_new;
 	Datum val_old;
@@ -1574,7 +1645,7 @@ ts_continuous_agg_get_query(ContinuousAgg *cagg)
  * Get the width of a fixed size bucket
  */
 int64
-ts_continuous_agg_fixed_bucket_width(const ContinuousAggsBucketFunction *bucket_function)
+ts_continuous_agg_fixed_bucket_width(const ContinuousAggBucketFunction *bucket_function)
 {
 	Assert(bucket_function->bucket_fixed_interval == true);
 
@@ -1594,7 +1665,7 @@ ts_continuous_agg_fixed_bucket_width(const ContinuousAggsBucketFunction *bucket_
  * Get the width of a bucket
  */
 int64
-ts_continuous_agg_bucket_width(const ContinuousAggsBucketFunction *bucket_function)
+ts_continuous_agg_bucket_width(const ContinuousAggBucketFunction *bucket_function)
 {
 	int64 bucket_width;
 
