@@ -6,11 +6,14 @@ set -e
 RUNNER=${1:-""}
 
 sed  -e '/<exclude_from_test>/,/<\/exclude_from_test>/d' \
+     -e 's! Disk: [0-9]\{1,\}kB!!' \
      -e 's! Memory: [0-9]\{1,\}kB!!' \
      -e 's! Memory Usage: [0-9]\{1,\}kB!!' \
      -e 's! Average  Peak Memory: [0-9]\{1,\}kB!!' \
      -e 's!ERROR:  permission denied for materialized view!ERROR:  permission denied for view!' \
      -e '/Heap Fetches: [0-9]\{1,\}/d' \
+     -e '/Buckets: [0-9]\{1,\}/d' \
+     -e '/Batches: [0-9]\{1,\}/d' \
      -e '/found [0-9]\{1,\} removable, [0-9]\{1,\} nonremovable row versions in [0-9]\{1,\} pages/d' | \
 grep -av 'DEBUG:  rehashing catalog cache id' | \
 grep -av 'DEBUG:  compacted fsync request queue from' | \
