@@ -146,7 +146,7 @@ policy_compression_add_internal(Oid user_rel_oid, Datum compress_after_datum,
 								Interval *default_schedule_interval,
 								bool user_defined_schedule_interval, bool if_not_exists,
 								bool fixed_schedule, TimestampTz initial_start,
-								const char *timezone, UseAccessMethod use_access_method)
+								const char *timezone)
 {
 	NameData application_name;
 	NameData proc_name, proc_schema, check_schema, check_name, owner;
@@ -284,11 +284,6 @@ policy_compression_add_internal(Oid user_rel_oid, Datum compress_after_datum,
 	ts_jsonb_add_int32(parse_state, POLICY_CONFIG_KEY_HYPERTABLE_ID, hypertable->fd.id);
 	validate_compress_after_type(dim, partitioning_type, compress_after_type);
 
-	if (use_access_method != USE_AM_NULL)
-		ts_jsonb_add_bool(parse_state,
-						  POL_COMPRESSION_CONF_KEY_USE_ACCESS_METHOD,
-						  use_access_method);
-
 	switch (compress_after_type)
 	{
 		case INTERVALOID:
@@ -375,7 +370,6 @@ policy_compression_add(PG_FUNCTION_ARGS)
 	text *timezone = PG_ARGISNULL(5) ? NULL : PG_GETARG_TEXT_PP(5);
 	char *valid_timezone = NULL;
 	Interval *created_before = PG_GETARG_INTERVAL_P(6);
-	UseAccessMethod use_access_method = PG_ARGISNULL(7) ? USE_AM_NULL : PG_GETARG_BOOL(7);
 
 	ts_feature_flag_check(FEATURE_POLICY);
 	TS_PREVENT_FUNC_IF_READ_ONLY();
@@ -408,8 +402,7 @@ policy_compression_add(PG_FUNCTION_ARGS)
 											 if_not_exists,
 											 fixed_schedule,
 											 initial_start,
-											 valid_timezone,
-											 use_access_method);
+											 valid_timezone);
 
 	if (!TIMESTAMP_NOT_FINITE(initial_start))
 	{
