@@ -99,13 +99,17 @@ compression_hypertable_create(Hypertable *ht, Oid owner, Oid tablespace_oid)
 }
 
 Oid
-compression_chunk_create(Chunk *src_chunk, Chunk *chunk, List *column_defs, Oid tablespace_oid)
+compression_chunk_create(Chunk *src_chunk, Chunk *chunk, List *column_defs, Oid tablespace_oid,
+						 CompressionSettings *settings)
 {
 	ObjectAddress tbladdress;
 	CatalogSecurityContext sec_ctx;
 	Datum toast_options;
-	static char *validnsps[] = HEAP_RELOPT_NAMESPACES;
-	CompressionSettings *settings = ts_compression_settings_get(src_chunk->hypertable_relid);
+#if PG18_LT
+	char *validnsps[] = HEAP_RELOPT_NAMESPACES;
+#else
+	const char *const validnsps[] = HEAP_RELOPT_NAMESPACES;
+#endif
 
 	Oid owner = ts_rel_get_owner(chunk->hypertable_relid);
 
