@@ -1404,7 +1404,7 @@ compression_settings_set_defaults(Hypertable *ht, CompressionSettings *settings,
 		settings->fd.orderby = obs.orderby;
 		settings->fd.orderby_desc = obs.orderby_desc;
 		settings->fd.orderby_nullsfirst = obs.orderby_nullsfirst;
-		add_orderby_sparse_index = true;
+		add_orderby_sparse_index = settings->fd.index != NULL;
 	}
 
 	if (ts_guc_auto_sparse_indexes && can_set_default_sparse_index(settings))
@@ -1449,7 +1449,7 @@ compression_settings_set_manually_for_alter(Hypertable *ht, CompressionSettings
 		settings->fd.orderby = obs.orderby;
 		settings->fd.orderby_desc = obs.orderby_desc;
 		settings->fd.orderby_nullsfirst = obs.orderby_nullsfirst;
-		add_orderby_sparse_index = true;
+		add_orderby_sparse_index = settings->fd.index != NULL;
 	}
 
 	if (!with_clause_options[AlterTableFlagIndex].is_default)
@@ -1501,7 +1501,7 @@ compression_settings_set_manually_for_create(Hypertable *ht, CompressionSettings
 		settings->fd.orderby = obs.orderby;
 		settings->fd.orderby_desc = obs.orderby_desc;
 		settings->fd.orderby_nullsfirst = obs.orderby_nullsfirst;
-		add_orderby_sparse_index = true;
+		add_orderby_sparse_index = settings->fd.index != NULL;
 	}
 
 	if (!with_clause_options[CreateTableFlagIndex].is_default)
