@@ -16,9 +16,11 @@
  * cagg_create.
  */
 #include <postgres.h>
+
 #include <access/reloptions.h>
 #include <access/sysattr.h>
 #include <access/xact.h>
+#include <access/xlogutils.h>
 #include <catalog/index.h>
 #include <catalog/indexing.h>
 #include <catalog/pg_namespace.h>
@@ -30,6 +32,9 @@
 #include <commands/tablespace.h>
 #include <commands/trigger.h>
 #include <commands/view.h>
+#include <executor/spi.h>
+#include <fmgr.h>
+#include <miscadmin.h>
 #include <nodes/makefuncs.h>
 #include <nodes/nodeFuncs.h>
 #include <nodes/nodes.h>
@@ -45,36 +50,40 @@
 #include <parser/parse_relation.h>
 #include <parser/parse_type.h>
 #include <parser/parsetree.h>
+#include <replication/logical.h>
+#include <replication/slot.h>
+#include <storage/lwlocknames.h>
 #include <utils/acl.h>
 #include <utils/builtins.h>
 #include <utils/catcache.h>
+#include <utils/elog.h>
+#include <utils/pg_lsn.h>
 #include <utils/rel.h>
+#include <utils/resowner.h>
 #include <utils/ruleutils.h>
+#include <utils/snapshot.h>
 #include <utils/syscache.h>
 #include <utils/typcache.h>
 
 #include "common.h"
+#include "config.h"
 #include "create.h"
 #include "finalize.h"
 #include "invalidation_threshold.h"
 
+#include "continuous_aggs/invalidation_multi.h"
 #include "debug_assert.h"
 #include "dimension.h"
-#include "errors.h"
 #include "extension_constants.h"
-#include "func_cache.h"
 #include "guc.h"
 #include "hypertable.h"
 #include "hypertable_cache.h"
 #include "invalidation.h"
-#include "options.h"
 #include "refresh.h"
 #include "time_utils.h"
-#include "timezones.h"
 #include "ts_catalog/catalog.h"
 #include "ts_catalog/continuous_agg.h"
 #include "ts_catalog/continuous_aggs_watermark.h"
-#include "utils.h"
 #include "with_clause/create_materialized_view_with_clause.h"
 
 static void create_cagg_catalog_entry(int32 matht_id, int32 rawht_id, const char *user_schema,
@@ -89,15 +98,41 @@ static void create_bucket_function_catalog_entry(int32 matht_id, Oid bucket_func
 static void cagg_create_hypertable(int32 hypertable_id, Oid mat_tbloid, const char *matpartcolname,
 								   int64 mat_tbltimecol_interval);
 static void cagg_add_trigger_hypertable(Oid relid, int32 hypertable_id);
-static void mattablecolumninfo_add_mattable_index(MatTableColumnInfo *matcolinfo, Hypertable *ht);
+static void mattablecolumninfo_add_mattable_index(MaterializationHypertableColumnInfo *matcolinfo,
+												  Hypertable *ht);
 static ObjectAddress create_view_for_query(Query *selquery, RangeVar *viewrel);
 static void fixup_userview_query_tlist(Query *userquery, List *tlist_aliases);
 static void cagg_create(const CreateTableAsStmt *create_stmt, ViewStmt *stmt, Query *panquery,
-						CAggTimebucketInfo *bucket_info, WithClauseResult *with_clause_options);
+						ContinuousAggTimeBucketInfo *bucket_info,
+						WithClauseResult *with_clause_options);
 
 #define MATPARTCOL_INTERVAL_FACTOR 10
 #define CAGG_INVALIDATION_TRIGGER "continuous_agg_invalidation_trigger"
 
+static bool
+has_continuous_aggs_attached(int32 raw_hypertable_id)
+{
+	ScanIterator iterator =
+		ts_scan_iterator_create(CONTINUOUS_AGG, AccessShareLock, CurrentMemoryContext);
+
+	iterator.ctx.index =
+		catalog_get_index(ts_catalog_get(), CONTINUOUS_AGG, CONTINUOUS_AGG_RAW_HYPERTABLE_ID_IDX);
+
+	ts_scan_iterator_scan_key_init(&iterator,
+								   Anum_continuous_agg_raw_hypertable_id_idx_raw_hypertable_id,
+								   BTEqualStrategyNumber,
+								   F_INT4EQ,
+								   Int32GetDatum(raw_hypertable_id));
+
+	ts_scanner_foreach(&iterator)
+	{
+		ts_scan_iterator_close(&iterator);
+		return true;
+	}
+	ts_scan_iterator_close(&iterator);
+	return false;
+}
+
 static void
 makeMaterializedTableName(char *buf, const char *prefix, int hypertable_id)
 {
@@ -111,11 +146,12 @@ makeMaterializedTableName(char *buf, const char *prefix, int hypertable_id)
 
 /* STATIC functions defined on the structs above. */
 static int32 mattablecolumninfo_create_materialization_table(
-	MatTableColumnInfo *matcolinfo, int32 hypertable_id, RangeVar *mat_rel,
-	CAggTimebucketInfo *bucket_info, bool create_addl_index, char *tablespacename,
+	MaterializationHypertableColumnInfo *matcolinfo, int32 hypertable_id, RangeVar *mat_rel,
+	ContinuousAggTimeBucketInfo *bucket_info, bool create_addl_index, char *tablespacename,
 	char *table_access_method, int64 matpartcol_interval, ObjectAddress *mataddress);
-static Query *mattablecolumninfo_get_partial_select_query(MatTableColumnInfo *mattblinfo,
-														  Query *userview_query, bool finalized);
+static Query *
+mattablecolumninfo_get_partial_select_query(MaterializationHypertableColumnInfo *mattblinfo,
+											Query *userview_query, bool finalized);
 
 /*
  * Create a entry for the materialization table in table CONTINUOUS_AGGS.
@@ -298,6 +334,91 @@ cagg_create_hypertable(int32 hypertable_id, Oid mat_tbloid, const char *matpartc
 				 errmsg("could not create materialization hypertable")));
 }
 
+/*
+ * Add a logical decoding slot for reading invalidations.
+ *
+ * It is the responsibility of the caller to check that the replication slot
+ * does not exist. There is an assert to check this.
+ *
+ * This code is mostly copied from create_logical_replication_slot and friends
+ * in slotfunc.c.
+ *
+ * It is important that the slot exists if and only if there are continuous
+ * aggregates that use the WAL to collect invalidations.
+ *
+ * Obviously, the slot is needed to read invalidations for the hypertables and
+ * it needs to have the version expected by the timescale extension. If the
+ * versions do not match, the rows returned by pg_logical_slot_get_changes()
+ * will not match what is expected and decoding will fail.
+ *
+ * Less obvious, the slot needs to be removed once there are no more continuous
+ * aggregates that need the slot. If not, the WAL will not be used and will
+ * start to fill up storage.
+ *
+ * There is a bug in PostgreSQL causing it to error out before the slot is
+ * created in LogicalDecodingProcessRecord() trying to allocate too much
+ * memory if you have too many other concurrent transactions in progress.
+ */
+static void
+cagg_add_logical_decoding_slot_prepare(const char *slot_name)
+{
+	CatalogSecurityContext sec_ctx;
+	LogicalDecodingContext *ctx = NULL;
+	const char *plugin_name = CONTINUOUS_AGGS_HYPERTABLE_INVALIDATION_PLUGIN_NAME;
+
+	Assert(!MyReplicationSlot);
+	Assert(SearchNamedReplicationSlot(slot_name, true) == NULL);
+
+	ts_catalog_database_info_become_owner(ts_catalog_database_info_get(), &sec_ctx);
+
+	TS_DEBUG_LOG("create invalidation slot %s", slot_name);
+
+	/*
+	 * Similar to how it works in create_logical_replication_slot(), this
+	 * initially create persistent slot as ephemeral to be able to handle
+	 * errors during initialization and allow it to be dropped if this
+	 * transaction fails. It's made persistent after the continuous aggregate
+	 * has been successfully created.
+	 */
+
+#if PG17_LT
+	ReplicationSlotCreate(slot_name, true, RS_EPHEMERAL, false);
+#else
+	ReplicationSlotCreate(slot_name, true, RS_EPHEMERAL, false, false, false);
+#endif
+	TS_DEBUG_LOG("init plugin %s", plugin_name);
+	ctx = CreateInitDecodingContext(plugin_name,
+									NIL,
+									false,
+									InvalidXLogRecPtr,
+									XL_ROUTINE(.page_read = read_local_xlog_page,
+											   .segment_open = wal_segment_open,
+											   .segment_close = wal_segment_close),
+									NULL,
+									NULL,
+									NULL);
+
+	/*
+	 * To avoid the above mentioned bug, we can find our own start point here
+	 * in a manner that does not require allocating a lot of memory.
+	 */
+	TS_DEBUG_LOG("find startpoint");
+	DecodingContextFindStartpoint(ctx);
+
+	TS_DEBUG_LOG("free decoding context");
+	FreeDecodingContext(ctx);
+
+	ts_catalog_restore_user(&sec_ctx);
+}
+
+static void
+cagg_add_logical_decoding_slot_finalize(void)
+{
+	TS_DEBUG_LOG("persist invalidation slot");
+	ReplicationSlotPersist();
+	ReplicationSlotRelease();
+}
+
 /*
  * Add continuous agg invalidation trigger to hypertable
  * relid - oid of hypertable
@@ -350,7 +471,8 @@ cagg_add_trigger_hypertable(Oid relid, int32 hypertable_id)
  * i.e. #indexes =(  #grp-cols - 1)
  */
 static void
-mattablecolumninfo_add_mattable_index(MatTableColumnInfo *matcolinfo, Hypertable *ht)
+mattablecolumninfo_add_mattable_index(MaterializationHypertableColumnInfo *matcolinfo,
+									  Hypertable *ht)
 {
 	IndexStmt stmt = {
 		.type = T_IndexStmt,
@@ -416,19 +538,21 @@ mattablecolumninfo_add_mattable_index(MatTableColumnInfo *matcolinfo, Hypertable
  *        materialization table
  */
 static int32
-mattablecolumninfo_create_materialization_table(MatTableColumnInfo *matcolinfo, int32 hypertable_id,
-												RangeVar *mat_rel, CAggTimebucketInfo *bucket_info,
-												bool create_addl_index, char *const tablespacename,
-												char *const table_access_method,
-												int64 matpartcol_interval,
-												ObjectAddress *mataddress)
+mattablecolumninfo_create_materialization_table(
+	MaterializationHypertableColumnInfo *matcolinfo, int32 hypertable_id, RangeVar *mat_rel,
+	ContinuousAggTimeBucketInfo *bucket_info, bool create_addl_index, char *const tablespacename,
+	char *const table_access_method, int64 matpartcol_interval, ObjectAddress *mataddress)
 {
 	Oid uid, saved_uid;
 	int sec_ctx;
 	char *matpartcolname = matcolinfo->matpartcolname;
 	CreateStmt *create;
 	Datum toast_options;
-	static char *validnsps[] = HEAP_RELOPT_NAMESPACES;
+#if PG18_LT
+	char *validnsps[] = HEAP_RELOPT_NAMESPACES;
+#else
+	const char *const validnsps[] = HEAP_RELOPT_NAMESPACES;
+#endif
 	int32 mat_htid;
 	Oid mat_relid;
 	Cache *hcache;
@@ -487,8 +611,8 @@ mattablecolumninfo_create_materialization_table(MatTableColumnInfo *matcolinfo,
  * the materialization columns and remove HAVING clause and ORDER BY.
  */
 static Query *
-mattablecolumninfo_get_partial_select_query(MatTableColumnInfo *mattblinfo, Query *userview_query,
-											bool finalized)
+mattablecolumninfo_get_partial_select_query(MaterializationHypertableColumnInfo *mattblinfo,
+											Query *userview_query, bool finalized)
 {
 	Query *partial_selquery = NULL;
 
@@ -596,6 +720,41 @@ fixup_userview_query_tlist(Query *userquery, List *tlist_aliases)
 	}
 }
 
+static const char *invalidate_using_info[] = {
+	[ContinuousAggInvalidateUsingDefault] = "default",
+	[ContinuousAggInvalidateUsingTrigger] = "trigger",
+	[ContinuousAggInvalidateUsingWal] = "wal",
+};
+
+static ContinuousAggInvalidateUsing
+get_invalidate_using(WithClauseResult *with_clause_options)
+{
+	if (with_clause_options[CreateMaterializedViewFlagInvalidateUsing].is_default)
+		return ContinuousAggInvalidateUsingDefault;
+
+	const char *invalidate_using = text_to_cstring(
+		DatumGetTextP(with_clause_options[CreateMaterializedViewFlagInvalidateUsing].parsed));
+
+	for (size_t i = 0; i < sizeof(invalidate_using_info) / sizeof(*invalidate_using_info); ++i)
+		if (strcmp(invalidate_using_info[i], invalidate_using) == 0)
+			return i;
+	ereport(ERROR,
+			errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+			errmsg("unrecognized value \"%s\" for invalidate_using", invalidate_using));
+	return -1; /* To keep linter happy */
+}
+
+static void
+error_not_in_prerequisite_state(const char *wanted, const char *used)
+{
+	ereport(ERROR,
+			errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+			errmsg("cannot use %s-based invalidation collection with hypertable that is using "
+				   "%s-based invalidation collection",
+				   wanted,
+				   used));
+}
+
 /*
  * Modifies the passed in ViewStmt to do the following
  * a) Create a hypertable for the continuous agg materialization.
@@ -653,11 +812,11 @@ fixup_userview_query_tlist(Query *userquery, List *tlist_aliases)
  */
 static void
 cagg_create(const CreateTableAsStmt *create_stmt, ViewStmt *stmt, Query *panquery,
-			CAggTimebucketInfo *bucket_info, WithClauseResult *with_clause_options)
+			ContinuousAggTimeBucketInfo *bucket_info, WithClauseResult *with_clause_options)
 {
 	ObjectAddress mataddress;
 	char relnamebuf[NAMEDATALEN];
-	MatTableColumnInfo mattblinfo;
+	MaterializationHypertableColumnInfo mattblinfo;
 	FinalizeQueryInfo finalqinfo;
 	CatalogSecurityContext sec_ctx;
 	bool is_create_mattbl_index;
@@ -671,6 +830,10 @@ cagg_create(const CreateTableAsStmt *create_stmt, ViewStmt *stmt, Query *panquer
 	bool materialized_only =
 		DatumGetBool(with_clause_options[CreateMaterializedViewFlagMaterializedOnly].parsed);
 	bool finalized = DatumGetBool(with_clause_options[CreateMaterializedViewFlagFinalized].parsed);
+	ContinuousAggInvalidateUsing invalidate_using = get_invalidate_using(with_clause_options);
+	bool trigger_exists = has_invalidation_trigger(bucket_info->htoid);
+	bool caggs_exists = has_continuous_aggs_attached(bucket_info->htid);
+	bool slot_prepared = false;
 
 	int64 matpartcol_interval = 0;
 	if (!with_clause_options[CreateMaterializedViewFlagChunkTimeInterval].is_default)
@@ -703,6 +866,49 @@ cagg_create(const CreateTableAsStmt *create_stmt, ViewStmt *stmt, Query *panquer
 	 */
 	stmt->options = NULL;
 
+	/*
+	 * If we want to create a continuous aggregate using WAL-based
+	 * invalidation collection, it should not have an invalidation trigger
+	 * already.
+	 */
+	if (invalidate_using == ContinuousAggInvalidateUsingWal && trigger_exists)
+		error_not_in_prerequisite_state("wal", "trigger");
+
+	/*
+	 * If we want to create a continuous aggregate with a trigger, the
+	 * hypertable should either have a trigger already, or if there is no
+	 * trigger, no hypertables should be attached to it.
+	 */
+	if (invalidate_using == ContinuousAggInvalidateUsingTrigger && !trigger_exists && caggs_exists)
+		error_not_in_prerequisite_state("trigger", "wal");
+
+	/*
+	 * If we didn't specify which collection method to use, we pick whatever
+	 * is used, or the default if there are no continuous aggregates attached.
+	 */
+	if (invalidate_using == ContinuousAggInvalidateUsingDefault)
+	{
+		if (!caggs_exists)
+			invalidate_using = ContinuousAggInvalidateUsingTrigger; /* default case */
+		else if (trigger_exists)
+			invalidate_using = ContinuousAggInvalidateUsingTrigger;
+		else
+			invalidate_using = ContinuousAggInvalidateUsingWal;
+	}
+
+	/*
+	 * Step 0: Create logical replication slot if used. This needs to be first
+	 * since we cannot do this in a transaction that has performed any writes.
+	 */
+	char slot_name[TS_INVALIDATION_SLOT_NAME_MAX];
+	ts_get_invalidation_replication_slot_name(slot_name, sizeof(slot_name));
+	if (invalidate_using == ContinuousAggInvalidateUsingWal &&
+		SearchNamedReplicationSlot(slot_name, true) == NULL)
+	{
+		cagg_add_logical_decoding_slot_prepare(slot_name);
+		slot_prepared = true;
+	}
+
 	/*
 	 * Old format caggs are not supported anymore, there is no need to add
 	 * an internal chunk id column for materialized hypertable.
@@ -725,6 +931,7 @@ cagg_create(const CreateTableAsStmt *create_stmt, ViewStmt *stmt, Query *panquer
 													create_stmt->into->accessMethod,
 													matpartcol_interval,
 													&mataddress);
+
 	/*
 	 * Step 2: Create view with select finalize from materialization table.
 	 */
@@ -833,8 +1040,12 @@ cagg_create(const CreateTableAsStmt *create_stmt, ViewStmt *stmt, Query *panquer
 										 bucket_info->bf->bucket_time_timezone,
 										 bucket_info->bf->bucket_fixed_interval);
 
-	/* Step 5: Create trigger on raw hypertable -specified in the user view query. */
-	cagg_add_trigger_hypertable(bucket_info->htoid, bucket_info->htid);
+	/* Step 5: Create trigger on raw hypertable, if needed, and finalize slot
+	 * creation if that needs to be done. */
+	if (invalidate_using == ContinuousAggInvalidateUsingTrigger)
+		cagg_add_trigger_hypertable(bucket_info->htoid, bucket_info->htid);
+	else if (slot_prepared)
+		cagg_add_logical_decoding_slot_finalize();
 }
 
 DDLResult
@@ -842,13 +1053,13 @@ tsl_process_continuous_agg_viewstmt(Node *node, const char *query_string, void *
 									WithClauseResult *with_clause_options)
 {
 	const CreateTableAsStmt *stmt = castNode(CreateTableAsStmt, node);
-	CAggTimebucketInfo timebucket_exprinfo;
+	ContinuousAggTimeBucketInfo timebucket_exprinfo;
 	Oid nspid;
 	bool finalized = with_clause_options[CreateMaterializedViewFlagFinalized].parsed;
 	ViewStmt viewstmt = {
 		.type = T_ViewStmt,
 		.view = stmt->into->rel,
-		.query = stmt->into->viewQuery,
+		.query = (Node *) stmt->into->viewQuery,
 		.options = stmt->into->options,
 		.aliases = stmt->into->colNames,
 	};
@@ -950,7 +1161,7 @@ tsl_process_continuous_agg_viewstmt(Node *node, const char *query_string, void *
 		refresh_window.start = cagg_get_time_min(cagg);
 		refresh_window.end = ts_time_get_noend_or_max(refresh_window.type);
 
-		CaggRefreshContext context = { .callctx = CAGG_REFRESH_CREATION };
+		ContinuousAggRefreshContext context = { .callctx = CAGG_REFRESH_CREATION };
 		continuous_agg_refresh_internal(cagg,
 										&refresh_window,
 										context,
@@ -996,7 +1207,7 @@ cagg_flip_realtime_view_definition(ContinuousAgg *agg, Hypertable *mat_ht)
 	relation_close(direct_view_rel, NoLock);
 	RemoveRangeTableEntries(direct_query);
 
-	CAggTimebucketInfo timebucket_exprinfo =
+	ContinuousAggTimeBucketInfo timebucket_exprinfo =
 		cagg_validate_query(direct_query,
 							agg->data.finalized,
 							NameStr(agg->data.user_view_schema),
