@@ -196,6 +196,7 @@ invalidation_threshold_get(int32 hypertable_id, Oid type)
 		.waitpolicy = LockWaitBlock,
 		.lockmode = LockTupleExclusive,
 	};
+	PushActiveSnapshot(GetLatestSnapshot());
 	ScannerCtx scanctx = {
 		.table = catalog_get_table_id(catalog, CONTINUOUS_AGGS_INVALIDATION_THRESHOLD),
 		.index =
@@ -209,7 +210,7 @@ invalidation_threshold_get(int32 hypertable_id, Oid type)
 		.result_mctx = CurrentMemoryContext,
 		.tuplock = &scantuplock,
 		.flags = SCANNER_F_KEEPLOCK,
-		.snapshot = GetLatestSnapshot(),
+		.snapshot = GetActiveSnapshot(),
 	};
 
 	ScanKeyInit(&scankey[0],
@@ -220,6 +221,7 @@ invalidation_threshold_get(int32 hypertable_id, Oid type)
 
 	bool found = ts_scanner_scan_one(&scanctx, false, CAGG_INVALIDATION_THRESHOLD_NAME);
 	Ensure(found, "invalidation threshold for hypertable %d not found", hypertable_id);
+	PopActiveSnapshot();
 	return data.threshold;
 }
 
@@ -247,6 +249,7 @@ invalidation_threshold_set_or_get(const ContinuousAgg *cagg,
 		.cagg = cagg,
 		.refresh_window = refresh_window,
 	};
+	PushActiveSnapshot(GetLatestSnapshot());
 	ScannerCtx scanctx = {
 		.table = catalog_get_table_id(catalog, CONTINUOUS_AGGS_INVALIDATION_THRESHOLD),
 		.index =
@@ -265,7 +268,7 @@ invalidation_threshold_set_or_get(const ContinuousAgg *cagg,
 		 * snapshot includes "changes made by the current command") and ts_scanner_scan_one()
 		 * would fail due to the second found tuple. A normal MVCC snapshot is used to prevent
 		 * the update is immediately seen by the scanner. */
-		.snapshot = GetLatestSnapshot(),
+		.snapshot = GetActiveSnapshot(),
 	};
 
 	ScanKeyInit(&scankey[0],
@@ -278,6 +281,7 @@ invalidation_threshold_set_or_get(const ContinuousAgg *cagg,
 	Ensure(found,
 		   "invalidation threshold for hypertable %d not found",
 		   cagg->data.raw_hypertable_id);
+	PopActiveSnapshot();
 
 	return updatectx.computed_invalidation_threshold;
 }
