@@ -697,12 +697,15 @@ process_cagg_invalidations_and_refresh(const ContinuousAgg *cagg,
 	 * This is supposed to be a short transaction and in the future we can consider
 	 * relaxing this lock.
 	 */
-	LockRelationOid(hyper_relid, ExclusiveLock);
+	LockRelationOid(hyper_relid, ShareUpdateExclusiveLock);
 	invalidations = invalidation_process_cagg_log(cagg,
 												  refresh_window,
 												  ts_guc_cagg_max_individual_materializations,
 												  context,
 												  force);
+
+	DEBUG_WAITPOINT("before_process_cagg_invalidations_for_refresh_lock");
+
 	SPI_commit_and_chain();
 
 	DEBUG_WAITPOINT("after_process_cagg_invalidations_for_refresh_lock");
@@ -919,13 +922,46 @@ continuous_agg_refresh_internal(const ContinuousAgg *cagg,
 
 	cagg = ts_continuous_agg_find_by_mat_hypertable_id(mat_id, false);
 
-	if (!process_cagg_invalidations_and_refresh(cagg,
-												&refresh_window,
-												context,
-												INVALID_CHUNK_ID,
-												force))
+	bool refreshed = process_cagg_invalidations_and_refresh(cagg,
+															&refresh_window,
+															context,
+															INVALID_CHUNK_ID,
+															force);
+
+	/* check if we have any pending materializations in our refresh window range,
+	 * if so, we need to process them
+	 * Note that we use the original refresh window range here, not the one that has been processed
+	 * by the refresh function*/
+	refresh_window = *refresh_window_arg;
+	bool has_pending_materializations =
+		continuous_agg_has_pending_materializations(cagg, refresh_window);
+
+	if (has_pending_materializations)
+	{
+		ContinuousAggRefreshState refresh;
+		continuous_agg_refresh_init(&refresh, cagg, &refresh_window);
+
+		InternalTimeRange invalidation = {
+			.type = refresh_window.type,
+			.start = refresh_window.start,
+			/* Invalidations are inclusive at the end, while refresh windows
+			 * aren't, so add one to the end of the invalidated region */
+			.end = ts_time_saturating_add(refresh_window.end, 1, refresh_window.type),
+		};
+
+		InternalTimeRange bucketed_refresh_window =
+			compute_circumscribed_bucketed_refresh_window(cagg,
+														  &invalidation,
+														  cagg->bucket_function);
+
+		continuous_agg_refresh_execute(&refresh, &bucketed_refresh_window, INVALID_CHUNK_ID);
+	}
+
+	if (!refreshed && !has_pending_materializations)
 		emit_up_to_date_notice(cagg, context);
 
+	DEBUG_WAITPOINT("after_process_cagg_materializations");
+
 	/* Restore search_path */
 	AtEOXact_GUC(false, save_nestlevel);
 
