@@ -737,8 +737,9 @@ get_direct_view_oid(int32 mat_hypertable_id)
 						   Int32GetDatum(mat_hypertable_id));
 
 	/* Prepare index scan */
+	PushActiveSnapshot(GetTransactionSnapshot());
 	IndexScanDesc indexscan =
-		index_beginscan(cagg_rel, cagg_idx_rel, GetTransactionSnapshot(), 1, 0);
+		index_beginscan_compat(cagg_rel, cagg_idx_rel, GetActiveSnapshot(), NULL, 1, 0);
 	index_rescan(indexscan, scankeys, 1, NULL, 0);
 
 	/* Read tuple from relation */
@@ -784,6 +785,7 @@ get_direct_view_oid(int32 mat_hypertable_id)
 	ExecDropSingleTupleTableSlot(slot);
 	relation_close(cagg_rel, AccessShareLock);
 	relation_close(cagg_idx_rel, AccessShareLock);
+	PopActiveSnapshot();
 
 	/* Get Oid of user view */
 	Oid direct_view_oid =
@@ -806,7 +808,7 @@ enum
 #define Natts_cagg_bucket_function (_Anum_cagg_bucket_function_max - 1)
 
 static Datum
-create_cagg_get_bucket_function_datum(TupleDesc tupdesc, ContinuousAggsBucketFunction *bf)
+create_cagg_get_bucket_function_datum(TupleDesc tupdesc, ContinuousAggBucketFunction *bf)
 {
 	NullableDatum datums[Natts_cagg_bucket_function] = { { 0 } };
 	HeapTuple tuple;
@@ -893,7 +895,7 @@ cagg_get_bucket_function_datum(int32 mat_hypertable_id, FunctionCallInfo fcinfo)
 	if (fcinfo != NULL && get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE)
 		elog(ERROR, "function returning record called in context that cannot accept type record");
 
-	ContinuousAggsBucketFunction *bf = ts_cagg_get_bucket_function_info(direct_view_oid);
+	ContinuousAggBucketFunction *bf = ts_cagg_get_bucket_function_info(direct_view_oid);
 
 	if (!OidIsValid(bf->bucket_function))
 	{
