@@ -22,29 +22,25 @@
 #include "compression/algorithms/deltadelta.h"
 #include "compression/algorithms/dictionary.h"
 #include "compression/algorithms/gorilla.h"
+#include "compression/algorithms/uuid_compress.h"
 #include "compression/api.h"
 #include "compression/compression.h"
 #include "compression/create.h"
 #include "compression/recompress.h"
 #include "compression/sparse_index_bloom1.h"
-#include "config.h"
 #include "continuous_aggs/create.h"
 #include "continuous_aggs/insert.h"
 #include "continuous_aggs/invalidation.h"
+#include "continuous_aggs/invalidation_multi.h"
+#include "continuous_aggs/invalidation_record.h"
 #include "continuous_aggs/options.h"
 #include "continuous_aggs/refresh.h"
 #include "continuous_aggs/repair.h"
 #include "continuous_aggs/utils.h"
 #include "cross_module_fn.h"
 #include "export.h"
-#include "hypercore/arrow_cache_explain.h"
-#include "hypercore/arrow_tts.h"
-#include "hypercore/attr_capture.h"
-#include "hypercore/hypercore_handler.h"
-#include "hypercore/hypercore_proxy.h"
 #include "hypertable.h"
 #include "license_guc.h"
-#include "nodes/columnar_scan/columnar_scan.h"
 #include "nodes/decompress_chunk/planner.h"
 #include "nodes/gapfill/gapfill_functions.h"
 #include "nodes/skip_scan/skip_scan.h"
@@ -66,12 +62,6 @@ PG_MODULE_MAGIC;
 extern void PGDLLEXPORT _PG_init(void);
 #endif
 
-static void
-tsl_xact_event(XactEvent event, void *arg)
-{
-	hypercore_xact_event(event, arg);
-}
-
 /*
  * Cross module function initialization.
  *
@@ -87,7 +77,6 @@ CrossModuleFunctions tsl_cm_functions = {
 	.create_upper_paths_hook = tsl_create_upper_paths_hook,
 	.set_rel_pathlist_dml = tsl_set_rel_pathlist_dml,
 	.set_rel_pathlist_query = tsl_set_rel_pathlist_query,
-	.process_explain_def = tsl_process_explain_def,
 
 	/* bgw policies */
 	.policy_compression_add = policy_compression_add,
@@ -157,6 +146,7 @@ CrossModuleFunctions tsl_cm_functions = {
 	.continuous_agg_get_bucket_function = continuous_agg_get_bucket_function,
 	.continuous_agg_get_bucket_function_info = continuous_agg_get_bucket_function_info,
 	.continuous_agg_migrate_to_time_bucket = continuous_agg_migrate_to_time_bucket,
+	.continuous_agg_read_invalidation_record = ts_invalidation_read_record,
 	.cagg_try_repair = tsl_cagg_try_repair,
 
 	/* Compression */
@@ -178,7 +168,10 @@ CrossModuleFunctions tsl_cm_functions = {
 	.array_compressor_finish = tsl_array_compressor_finish,
 	.bool_compressor_append = tsl_bool_compressor_append,
 	.bool_compressor_finish = tsl_bool_compressor_finish,
+	.uuid_compressor_append = tsl_uuid_compressor_append,
+	.uuid_compressor_finish = tsl_uuid_compressor_finish,
 	.bloom1_contains = bloom1_contains,
+	.bloom1_get_hash_function = bloom1_get_hash_function,
 	.process_compress_table = tsl_process_compress_table,
 	.process_altertable_cmd = tsl_process_altertable_cmd,
 	.process_rename_cmd = tsl_process_rename_cmd,
@@ -187,18 +180,12 @@ CrossModuleFunctions tsl_cm_functions = {
 	.decompress_batches_for_insert = decompress_batches_for_insert,
 	.init_decompress_state_for_insert = init_decompress_state_for_insert,
 	.decompress_target_segments = decompress_target_segments,
-	.hypercore_handler = hypercore_handler,
-	.hypercore_proxy_handler = hypercore_proxy_handler,
-	.hypercore_decompress_update_segment = hypercore_decompress_update_segment,
-	.is_compressed_tid = tsl_is_compressed_tid,
 	.compression_enable = tsl_compression_enable,
 	.compressor_init = tsl_compressor_init,
 	.compressor_add_slot = tsl_compressor_add_slot,
 	.compressor_flush = tsl_compressor_flush,
 	.compressor_free = tsl_compressor_free,
 	.compression_chunk_create = tsl_compression_chunk_create,
-	.ddl_command_start = tsl_ddl_command_start,
-	.ddl_command_end = tsl_ddl_command_end,
 	.show_chunk = chunk_show,
 	.create_compressed_chunk = tsl_create_compressed_chunk,
 	.create_chunk = chunk_create,
@@ -218,7 +205,6 @@ static void
 ts_module_cleanup_on_pg_exit(int code, Datum arg)
 {
 	_continuous_aggs_cache_inval_fini();
-	UnregisterXactCallback(tsl_xact_event, NULL);
 }
 
 TS_FUNCTION_INFO_V1(ts_module_init);
@@ -233,9 +219,6 @@ ts_module_init(PG_FUNCTION_ARGS)
 
 	_continuous_aggs_cache_inval_init();
 	_decompress_chunk_init();
-	_columnar_scan_init();
-	_arrow_cache_explain_init();
-	_attr_capture_init();
 	_skip_scan_init();
 	_vector_agg_init();
 
@@ -243,7 +226,6 @@ ts_module_init(PG_FUNCTION_ARGS)
 	if (register_proc_exit)
 		on_proc_exit(ts_module_cleanup_on_pg_exit, 0);
 
-	RegisterXactCallback(tsl_xact_event, NULL);
 	PG_RETURN_BOOL(true);
 }
 
