@@ -15,8 +15,6 @@ SELECT _timescaledb_internal.cagg_watermark_materialized(0);
 SELECT _timescaledb_internal.calculate_chunk_interval(0,0,0);
 SELECT _timescaledb_internal.chunk_constraint_add_table_constraint(NULL);
 SELECT _timescaledb_internal.chunk_id_from_relid(0);
-SELECT _timescaledb_internal.chunk_index_clone(0);
-SELECT _timescaledb_internal.chunk_index_replace(0,0);
 SELECT _timescaledb_internal.chunk_status(0);
 SELECT _timescaledb_internal.chunks_local_size(NULL,NULL);
 SELECT _timescaledb_internal.compressed_chunk_local_stats(NULL,NULL);
@@ -34,7 +32,6 @@ SELECT pg_typeof(_timescaledb_internal.get_os_info());
 SELECT _timescaledb_internal.get_partition_for_key(NULL::text);
 SELECT _timescaledb_internal.get_partition_hash(NULL::text);
 SELECT _timescaledb_internal.hypertable_local_size(NULL,NULL);
-SELECT _timescaledb_internal.indexes_local_size(NULL,NULL);
 SELECT _timescaledb_internal.interval_to_usec(NULL);
 SELECT _timescaledb_internal.partialize_agg(NULL::text);
 SELECT _timescaledb_internal.policy_compression_check(NULL);
