@@ -1009,3 +1009,74 @@ ON CONFLICT DO NOTHING;
 
 DROP TABLE t_1sec;
 
+-- regression test for SDC 3210
+
+CREATE TABLE gen_column(
+	"timestamp"                         timestamp with time zone NOT NULL,
+	device_id                           text,
+	device_id_generated 				text GENERATED ALWAYS AS (device_id) STORED,
+	speed                               double precision
+);
+
+ALTER TABLE gen_column ADD CONSTRAINT unique_device_timestamp_sensor
+	UNIQUE (device_id_generated, "timestamp");
+
+CREATE INDEX gen_column_timestamp_idx
+	ON gen_column
+	USING btree ("timestamp" DESC);
+
+SELECT create_hypertable(
+	relation => 'gen_column',
+	time_column_name => 'timestamp',
+	chunk_time_interval => interval '06:00:00',
+	create_default_indexes => false
+);
+
+ALTER TABLE gen_column SET (
+	timescaledb.compress,
+	timescaledb.compress_segmentby = 'device_id_generated',
+	timescaledb.compress_orderby='"timestamp"'
+);
+
+insert into gen_column ( "timestamp", device_id, speed) values
+( '2025-06-06 10:00:00', 'device_id_1', 100),
+( '2025-06-06 10:01:00', 'device_id_2', 100),
+( '2025-06-06 10:02:00', 'device_id_3', 100);
+
+SELECT compress_chunk(show_chunks('gen_column'));
+
+insert into gen_column ( "timestamp", device_id, speed) values
+( '2025-06-06 10:03:00', 'device_id_1', 100),
+( '2025-06-06 10:00:00', 'device_id_1', 110),
+( '2025-06-06 10:00:00', 'device_id_1', 110)
+ON CONFLICT DO NOTHING;
+
+-- should contain 2 rows for device_1 and one per any other device
+SELECT device_id_generated, count(*)
+FROM gen_column
+GROUP BY 1
+ORDER BY 1;
+
+SELECT decompress_chunk(show_chunks('gen_column'));
+
+ALTER TABLE gen_column SET (
+	timescaledb.compress,
+	timescaledb.compress_segmentby = '',
+	timescaledb.compress_orderby='"timestamp"'
+);
+SELECT compress_chunk(show_chunks('gen_column'));
+
+insert into gen_column ( "timestamp", device_id, speed) values
+( '2025-06-06 10:04:00', 'device_id_1', 100),
+( '2025-06-06 10:00:00', 'device_id_1', 110),
+( '2025-06-06 10:00:00', 'device_id_1', 110)
+ON CONFLICT DO NOTHING;
+
+-- should contain 3 rows for device_1 and one per any other device
+SELECT device_id_generated, count(*)
+FROM gen_column
+GROUP BY 1
+ORDER BY 1;
+
+DROP TABLE gen_column;
+
