@@ -62,3 +62,26 @@ explain (costs off) select distinct on (device) device, time from :CHUNK order b
 
 RESET timescaledb.enable_skipscan_for_distinct_aggregates;
 drop table ht_metrics_compressed;
+
+-- Fix for issue #8681: IndexScan is not chosen for columnstore segmented on varchar column
+-- We should chose IndexScan now, and use SkipScan as well
+CREATE TABLE record (time timestamptz not null, data varchar);
+SELECT table_name FROM create_hypertable('record','time');
+ALTER TABLE record SET (timescaledb.compress,timescaledb.compress_orderby='time desc', timescaledb.compress_segmentby='data');
+
+INSERT INTO record
+SELECT time, (array['Yes', 'No', 'Maybe'])[floor(random() * 3 + 1)]
+FROM generate_series('2000-01-01'::timestamptz,'2000-01-03'::timestamptz, '10 minute'::interval) AS g1(time);
+
+analyze record;
+SELECT compress_chunk(ch) FROM show_chunks('record') ch;
+
+-- enable_seqscan is OFF, should see IndexScan
+explain (buffers off, costs off) SELECT * FROM record ORDER BY data;
+-- SkipScan is chosen because IndexScan is chosen
+explain (buffers off, costs off) SELECT DISTINCT ON(data) * FROM record;
+-- (seg_col = const) condition is checked even when seg_col is coerced
+explain (buffers off, costs off) SELECT * FROM record WHERE data='Yes' ORDER BY data;
+explain (buffers off, costs off) SELECT * FROM record WHERE 'Yes' <= data ORDER BY data;
+
+drop table record cascade;
