@@ -5,6 +5,9 @@
 CREATE INDEX ON :TABLE(dev);
 CREATE INDEX ON :TABLE(time);
 
+-- To test scenarios with SkipScan/no SkipScan over a mix of uncompressed and compressed chunks
+SET timescaledb.enable_compressed_skipscan TO false;
+
 -- SkipScan with ordered append
 :PREFIX SELECT DISTINCT ON (time) time FROM :TABLE ORDER BY time;
 
@@ -25,3 +28,5 @@ DROP INDEX _timescaledb_internal._hyper_1_1_chunk_skip_scan_ht_dev_idx;
 
 -- IndexPath without pathkeys doesnt use SkipScan
 EXPLAIN (costs off, timing off, summary off) SELECT DISTINCT 1 FROM pg_rewrite;
+
+RESET timescaledb.enable_compressed_skipscan;
